/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 *
 * Copyright 2001 DAX Interactive, LLC.
 * 
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-plugins/src/org/opendj/plugins/rerun/player/RerunStreamSource.java,v 1.1.1.1 2001/09/29 20:36:57 jonathan Exp $
 * $Log: RerunStreamSource.java,v $
 * Revision 1.1.1.1  2001/09/29 20:36:57  jonathan
 * Imported opendj-plugins
 *
 */
package org.opendj.plugins.rerun.player;

import java.io.File;
import java.util.Date;
import org.jrev.MP3FileSource;
import org.opendj.util.Util;
import org.opendj.util.debug;
import org.opendj.beans.archive.AudioArchiveBean;

public class RerunStreamSource extends MP3FileSource {

    public static final String DEADAIR_PREFIX = "dead";
    public static final String DEADAIR_SUFFIX = ".mp3";    

    private AudioArchiveBean itsArchive = null;

    public RerunStreamSource ( AudioArchiveBean theArchive ) {
        super(theArchive.getShowfile());
        itsArchive = theArchive;
    }

    private RerunStreamSource ( String theMP3File, 
                                AudioArchiveBean theArchive ) {
        super(theMP3File);
        itsArchive = theArchive;
    }

    public RerunStreamSource getDeadAirStream ( String theMP3Dir ) {
        String aDeadAirMP3
            = theMP3Dir
            + File.separator
            + DEADAIR_PREFIX
            + itsArchive.getBitrate()
            + DEADAIR_SUFFIX
            ;
        File aDeadAirFile = new File(aDeadAirMP3);
        if ( !aDeadAirFile.exists() ) {
            // XXX should log this!
            log("dead air file NOT FOUND... (bad)");
            return null;
        }
        return new RerunStreamSource(aDeadAirMP3, itsArchive);
    }

    public int bitrate () { return itsArchive.getBitrate(); }

    public String album () {
        return Util.EMPTY_STRING;
    }
    
    /*
    public int read(byte[] inByteArray) {
        System.out.println("RerunStreamSource.read called (Showfile=" + itsArchive.getShowfile() + ")");
        int rval = super.read(inByteArray);
        System.out.println("RerunStreamSource.read returning " + rval);
        return rval;
    }
    */

    public String artist () {
        return itsArchive.getDjname();
    }

    public String title () {
        return itsArchive.getShowname();
    }

    public String track () {
        return Util.EMPTY_STRING;
    }

    private void log (String s) {
        debug.out((new Date()).toString() + ": RerunStreamSource: " + s);
    }
}
