/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 *
 * Copyright 2001 DAX Interactive, LLC.
 * 
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-plugins/src/org/opendj/plugins/rerun/player/RerunStreamer.java,v 1.1.1.1 2001/09/29 20:36:57 jonathan Exp $
 * $Log: RerunStreamer.java,v $
 * Revision 1.1.1.1  2001/09/29 20:36:57  jonathan
 * Imported opendj-plugins
 *
 */
package org.opendj.plugins.rerun.player;

import org.jrev.JRev;
import org.jrev.Logger;
import java.io.File;
import java.io.IOException;
import org.opendj.util.Util;
import org.opendj.util.Config;
import org.opendj.util.debug;
import org.opendj.util.log.Logger;
import org.opendj.beans.onair.OnAirBean;
import org.opendj.beans.archive.AudioArchiveBean;

public class RerunStreamer implements Runnable {

    public static final long SLEEP_INTERVAL = 1000;

    public static final boolean IS_PUBLIC      = true;
    public static final boolean NOT_PUBLIC     = false;
    public static final boolean ICY_HEADERS    = true;
    public static final boolean NO_ICY_HEADERS = false;

    private RerunHandler itsHandler = null;
    private OnAirBean itsOnAirBean = null;
    private AudioArchiveBean itsArchive = null;

    private volatile Thread itsMainThread = null;
    private volatile boolean isDone = false;
    private volatile boolean itsIsStreaming = false;

    private String ICECAST_IP       = null;
    private int    ICECAST_PORT     = -1;
    private String ICECAST_PASSWORD = null;
    private String LOG_DIR          = null;

    private JRev itsJRev = null;
    private RerunPlaylist itsPlaylist = null;
    private org.opendj.util.log.Logger LOG = null;

    public RerunStreamer (RerunHandler theHandler) {

        itsHandler   = theHandler;

        Config aConfig = Config.global();
        ICECAST_IP       = aConfig.getProperty(Config.PROP_ICECAST_IP);
        ICECAST_PORT     = Integer.parseInt(aConfig.getProperty(Config.PROP_ICECAST_PORT));
        ICECAST_PASSWORD = aConfig.getProperty(Config.PROP_ICECAST_PASSWORD);
        LOG_DIR          = aConfig.getProperty(Config.PROP_LOGDIR);
    }

    public void run () {

        itsMainThread = Thread.currentThread();
        isDone = false;
        itsIsStreaming = false;

        // Create the playlist
        itsPlaylist = new RerunPlaylist();
        try {
            itsOnAirBean = itsHandler.getOnAirBean();

            // This had better work...
            itsArchive = itsHandler.loadArchive(itsOnAirBean);
            if ( itsArchive == null ) {
                debug.out("RerunStreamer.constructor: blargh!!! itsArchive is NULL!");
                return;
            }
            
            debug.out("RerunStreamer: creating playlist");
            itsPlaylist.setArchive(itsArchive);

            // Create the JRev engine
            debug.out("RerunStreamer: creating JRev");
            itsJRev = new JRev( itsOnAirBean.getshowname(),
                                Util.EMPTY_STRING,
                                Util.EMPTY_STRING,
                                itsOnAirBean.getgenreval(),
                                itsPlaylist.getBitrate(),
                                ICECAST_IP,
                                ICECAST_PORT,
                                ICECAST_PASSWORD,
                                Util.stripUpTo(itsOnAirBean.getchannelpath(), "/"),
                                NOT_PUBLIC,
                                ICY_HEADERS,
                                null,
                                null,
                                new org.jrev.Logger(false, getLogPath(), 999),
                                itsPlaylist );
            debug.out("RerunStreamer: starting JRev");
            // itsJRev.start();
            itsIsStreaming = true;
            while ( !itsJRev.isRunning() ) {
                Util.sleep(250);
            }
            do {
                Util.sleep(SLEEP_INTERVAL);
            } while ( !isDone && itsJRev.isRunning() );

        } catch ( Exception e ) {
            debug.out("Exception in RerunStreamer.run: " + Util.getStackTrace(e));
            LOG.capture("Exception in RerunStreamer.run", e, org.opendj.util.log.Logger.L_ERROR);

        } finally {
            debug.out("RerunStreamer: in finally clause");
            cleanup();
        }
    }

    public void finish() {
        isDone = true;
        if ( itsMainThread != null ) {
            itsMainThread.interrupt();
            itsMainThread = null;
        }
    }

    public boolean isStreaming () { return itsIsStreaming; }

    private void cleanup() {
        itsIsStreaming = false; 
        try { itsJRev.shutdown(); } catch ( Exception e ) {}
        try { itsPlaylist.finish(); } catch ( Exception e ) {}
    }

    private String getLogPath () throws IOException {
        return File.createTempFile("arch" 
                                   + itsArchive.getArchid() 
                                   + "_", ".log", 
                                   new File(LOG_DIR)).getAbsolutePath(); 
    }
}
