/*
 * This file and everything in this package is covered under
 * the GNU General Public License.  See the LICENSE file at the top
 * level of this package for full details.  A copy of the license
 * is also available at: http://opendj.org/docs/licenses/LICENSE.gpl
 *
 * Copyright 2001 DAX Interactive, LLC.
 * 
 * $Author: jonathan $
 * $Revision: 1.1.1.1 $
 * $Header: /home/cvspublic/opendj/opendj-plugins/src/org/opendj/plugins/rerun/scheduler/RerunSchedulerStrategy.java,v 1.1.1.1 2001/09/29 20:36:57 jonathan Exp $
 * $Log: RerunSchedulerStrategy.java,v $
 * Revision 1.1.1.1  2001/09/29 20:36:57  jonathan
 * Imported opendj-plugins
 *
 */
package org.opendj.plugins.rerun.scheduler;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Iterator;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import org.opendj.util.Util;
import org.opendj.util.DBUtil;
import org.opendj.util.Config;
import org.opendj.util.debug;
import org.opendj.util.SchedBlock;
import org.opendj.beans.archive.AudioArchiveBean;

public class RerunSchedulerStrategy {

    public static final Long SZ_A = new Long(Util.MILLIS_IN_MINUTE * 30);
    public static final Long SZ_B = new Long(SZ_A.longValue() * 2);
    public static final Long SZ_C = new Long(SZ_A.longValue() * 3);
    public static final Long SZ_D = new Long(SZ_A.longValue() * 4);
    public static final Long SZ_E = new Long(SZ_A.longValue() * 5);
    public static final Long SZ_F = new Long(SZ_A.longValue() * 6);
    public static final Long SZ_G = new Long(SZ_A.longValue() * 7);
    public static final Long SZ_H = new Long(SZ_A.longValue() * 8);

    public static final Long[] SZ_ARRAY = { SZ_A, SZ_B, SZ_C, SZ_D, SZ_E, SZ_F, SZ_G, SZ_H };

    private final SlotCount[][] SZ_A_STRATEGIES = 
    { { new SlotCount(SZ_A, 1) } };
    
    private final SlotCount[][] SZ_B_STRATEGIES = 
    { 
        { new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_B, 1) }
    };

    private final SlotCount[][] SZ_C_STRATEGIES = 
    { 
        { new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_C, 1) }
    };

    private final SlotCount[][] SZ_D_STRATEGIES = 
    { 
        { new SlotCount(SZ_A, 4) },
        { new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_B, 2) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_D, 1) }
    };

    private final SlotCount[][] SZ_E_STRATEGIES = 
    { 
        { new SlotCount(SZ_A, 5) },
        { new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_B, 2), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_B, 1) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_E, 1) }
    };

    private final SlotCount[][] SZ_F_STRATEGIES = 
    { 
        { new SlotCount(SZ_A, 6) },
        { new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 4) },
        { new SlotCount(SZ_B, 2), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_B, 3) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_C, 2) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_B, 1) },
        { new SlotCount(SZ_E, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_F, 1) }
    };

    private final SlotCount[][] SZ_G_STRATEGIES = 
    {
        { new SlotCount(SZ_A, 7) },
        { new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 5) },
        { new SlotCount(SZ_B, 2), new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_B, 3), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_A, 4) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_B, 2) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_C, 2), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_E, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_E, 1), new SlotCount(SZ_B, 1) },
        { new SlotCount(SZ_F, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_G, 1) }
    };

    private final SlotCount[][] SZ_H_STRATEGIES = 
    { 
        { new SlotCount(SZ_A, 8) },
        { new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 6) },
        { new SlotCount(SZ_B, 2), new SlotCount(SZ_A, 4) },
        { new SlotCount(SZ_B, 3), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_B, 4) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_A, 5) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_C, 1), new SlotCount(SZ_B, 2), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_C, 2), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_C, 2), new SlotCount(SZ_B, 1) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_A, 4) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_D, 1), new SlotCount(SZ_B, 2) },
        { new SlotCount(SZ_D, 2) },
        { new SlotCount(SZ_E, 1), new SlotCount(SZ_A, 3) },
        { new SlotCount(SZ_E, 1), new SlotCount(SZ_B, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_E, 1), new SlotCount(SZ_C, 1) },
        { new SlotCount(SZ_F, 1), new SlotCount(SZ_A, 2) },
        { new SlotCount(SZ_F, 1), new SlotCount(SZ_B, 1) },
        { new SlotCount(SZ_G, 1), new SlotCount(SZ_A, 1) },
        { new SlotCount(SZ_H, 1) }
    };

    private final SlotCount[][][] SZ_STRATEGIES_ARRAY = 
    { SZ_A_STRATEGIES, 
      SZ_B_STRATEGIES, 
      SZ_C_STRATEGIES, 
      SZ_D_STRATEGIES, 
      SZ_E_STRATEGIES, 
      SZ_F_STRATEGIES, 
      SZ_G_STRATEGIES, 
      SZ_H_STRATEGIES };

    private Map    itsStrategies  = null;
    private String itsRerunIPAddr = null;
    private int    itsRerunDjid   = -1;

    public RerunSchedulerStrategy () {

        itsStrategies = new HashMap();
        for ( int i=0; i<SZ_ARRAY.length; i++ ) {
            itsStrategies.put( SZ_ARRAY[i], SZ_STRATEGIES_ARRAY[i] );
        }
        Config aConfig = Config.global();
        itsRerunIPAddr = aConfig.getProperty(Config.PROP_RERUNIP);
        itsRerunDjid   = Integer.parseInt(aConfig.getProperty(Config.PROP_RERUNDJID));
    }

    public void fillSlotWithReruns ( Connection theConn, 
                                     EmptySlot theSlot, 
                                     RerunScheduler theScheduler ) throws SQLException {

        // Find the appropriate strategy for a slot of this size 
        Long aDurationLong = new Long(theSlot.getDuration());
        SlotCount[][] someStrategies = (SlotCount[][]) itsStrategies.get(aDurationLong);
        if ( someStrategies == null ) {
            // Irregular slot, bail out
            debug.out("RerunSchedulerStrategy.fillSlotWithReruns: "
                      + "Irregular empty slot duration (" + aDurationLong.longValue() + "), "
                      + "slot=" + theSlot + ", returning...");
            return;
        }

        // Iterate over strategies in REVERSE order, until we 
        // find something that works
        SlotCount[] aStrategy = null;
        for ( int tries=0; tries<2; tries++ ) {
            boolean doRegeneration = (tries == 1);

            for ( int i=someStrategies.length-1; i>=0; i-- ) {

                aStrategy = someStrategies[i];
                debug.out("fillSlotWithReruns: trying strategy="
                          + aStrategy + ", length=" + aStrategy.length);

                // Make sure the scheduler has the right available reruns to handle this...
                boolean aStrategyIsFeasible = true;
                for ( int j=0; j<aStrategy.length; j++ ) {
                    if ( !theScheduler.hasEnoughReruns( aStrategy[j].size.longValue(), 
                                                        aStrategy[j].count,
                                                        doRegeneration ) ) {
                        aStrategyIsFeasible = false;
                        break;
                    }
                }
                
                // If this strategy was not feasible, try the next one
                if ( !aStrategyIsFeasible ) continue;

                // Apply the strategy that was found to be feasible
                debug.out("fillSlotWithReruns: using strategy=" + i
                          + " for slot of size=" + aDurationLong.longValue());
                applyStrategy(theConn, theSlot, theScheduler, aStrategy);
                return;
            }
        }

        // No strategy found... shrink the slot by 1/2 hour and try to fill that time, if we can
        if ( theSlot.getDuration() > SZ_A.longValue() ) {
            debug.out("Warning: no strategy found that fills slot: " + theSlot + ", trying shorter slot");
            theSlot = new EmptySlot(theSlot.getStart(), theSlot.getEnd() - SZ_A.longValue());
            fillSlotWithReruns(theConn, theSlot, theScheduler);
        }
        debug.out("Warning: no strategy found that fills slot: " + theSlot);
    }

    private void applyStrategy ( Connection theConn, 
                                 EmptySlot theSlot, 
                                 RerunScheduler theScheduler,
                                 SlotCount[] theStrategy ) throws SQLException {
        debug.out("applyStrategy: filling slot: " + theSlot);
        AudioArchiveBean aRerun = null;
        List someReruns = new Vector();
        int i=0; int j=0;

        // For each SlotCount in the strategy, get the proper number of reruns from the scheduler
        for ( i=0; i<theStrategy.length; i++ ) {
            for ( j=0; j<theStrategy[i].count; j++ ) {
                aRerun = theScheduler.getRerunOfLength(theStrategy[i].size.longValue());
                someReruns.add(aRerun);
            }
        }

        // Shuffle them up
        Collections.shuffle(someReruns);

        // Finally, iterate over someReruns, and schedule each one
        Iterator iter = someReruns.iterator();
        
        // Do the entire rerun scheduling in a transaction
        PreparedStatement aPS = null;
        String aSQL 
            = "INSERT INTO PublishedSchedule "
            + "( channelid, channelpath, channelname, djid, edjid, ipaddr, djemail, "
            + "djname, showname, genre, genreval, starttime, endtime, numweeks, "
            + "archid, status ) VALUES "
            + "( ?, (SELECT path FROM Channel WHERE cid=?), "
            + "(SELECT name FROM Channel WHERE cid=?), "
            + " ?, ?, ?, (SELECT email FROM Dj WHERE djid=?), "
            + " ?, ?, ?, ?, ?, ?, ?, ?, ? ) "
            ;
        int aChannelid = -1;
        int aDjid = -1;
        long aStartTime = -1;
        long aDuration = -1;
        long anEndTime = -1;
        try {
            aStartTime = theSlot.getStart();
            theConn.setAutoCommit(false);
            aPS = theConn.prepareStatement(aSQL);

            while ( iter.hasNext() ) {
                aRerun = (AudioArchiveBean) iter.next();
                i=1;
                aChannelid = aRerun.getChannelid();
                aDjid = aRerun.getDjid();
                aDuration = aRerun.getScheduledDuration();
                anEndTime = aStartTime + aDuration;

                // Sanity check
                if ( anEndTime > theSlot.getEnd() ) {
                    throw new IllegalStateException("Total size of reruns selected are larger than "
                                                    + "the size of the empty slot to be filled! "
                                                    + "Slot=" + theSlot+ ", strategy=" + theStrategy);
                }
                
                aPS.setInt   (i++, aChannelid);           // channelid
                aPS.setInt   (i++, aChannelid);           // channelpath
                aPS.setInt   (i++, aChannelid);           // channelname
                aPS.setInt   (i++, itsRerunDjid);         // djid is RERUN DJ
                aPS.setInt   (i++, aDjid);                // edjid is original DJ
                aPS.setString(i++, itsRerunIPAddr);       // rerun IP addr
                aPS.setInt   (i++, aDjid);                // djemail
                aPS.setString(i++, aRerun.getDjname());   // djname
                aPS.setString(i++, aRerun.getShowname()); // showname
                aPS.setInt   (i++, aRerun.getGenre());    // genre
                aPS.setString(i++, aRerun.getGenreval()); // genreval
                aPS.setLong  (i++, aStartTime);           // starttime
                aPS.setLong  (i++, anEndTime);            // endtime
                aPS.setInt   (i++, 0);                    // numweeks
                aPS.setInt   (i++, aRerun.getArchid());   // archid
                aPS.setInt   (i++, SchedBlock.STATUS_RS_SCHEDULED_RERUN);// status

                aPS.executeUpdate();

                aStartTime = anEndTime;
            }

            debug.out("All reruns successfully scheduled...");
            theConn.commit();

        } catch ( Exception e ) {
            theConn.rollback();
            if ( e instanceof SQLException ) throw (SQLException) e;
            throw new SQLException(e.toString() + "\n" + Util.getStackTrace(e));
            
        } finally {
            DBUtil.closeStatement(aPS);
        }
    }

    public class SlotCount {

        public Long size;
        public int count;

        public SlotCount ( Long theSize, int theCount ) {
            size = theSize;
            count = theCount;
        }

        public String toString () {
            return "[SlotCount size=" + size.longValue() + ", count=" + count + "]";
        }
    }

}
